/*****************************************************************
 * Copyright (c) 2015 Texas Instruments and others
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Raymond Pang - Initial API and implementation
 *****************************************************************/
var gc = gc || {};
gc.databind = gc.databind || {};


(function() // closure for private static methods and data.
{
	var g_mqttViewInstance;
	
    var MqttTopicBind = function(name)
    {
    	gc.databind.VariableBindStreamingValue.call(this);
    	this.setStale(true);
    };
    
    MqttTopicBind.prototype = new gc.databind.VariableBindStreamingValue();

    MqttTopicBind.prototype.onValueChanged = function(newValue, oldValue,progress)
    {
        this.sendValue( newValue , progress);
    };
    
 	MqttTopicBind.prototype.sendValue = function(value, progress) 
    {
        var jsonString = JSON.stringify(value);
        if (g_mqttViewInstance)
        {
        	var topic = this.getName();
        	var pos = topic.indexOf('.');
        	if ( pos != -1 )
        		topic = topic.substring(pos+1);
        	g_mqttViewInstance.sendTopic( topic, jsonString);
        }
        else 
        {
            console.log('Discarded tx mqtt data: ' + jsonString);
        }
    };

	var MqttModel = function() 
	{
	  	gc.databind.AbstractStreamingDataModel.call(this);
	};
	
	gc.databind.MqttModel = 
	{
		setMqttViewInstance: function(mqttViewInstance) 
		{
			g_mqttViewInstance = mqttViewInstance;
		}
	};
	
	MqttModel.prototype = new gc.databind.AbstractStreamingDataModel('mqtt');
	
	MqttModel.prototype.createNewBind = function(name)
	{
		if (g_mqttViewInstance  && g_mqttViewInstance.isTopicName( name ) )
		{
			return new MqttTopicBind(name);
		}
		else
		{
			return gc.databind.AbstractStreamingDataModel.prototype.createNewBind.call(this, name);
		}
	};
	
	// register program model with bindingRegistry
	gc.databind.registry.registerModel(new gc.databind.StorageProviderBindFactoryWrapper(new MqttModel()), true);
	
}());






