/*****************************************************************
 * Copyright (c) 2015 Texas Instruments and others
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Raymond Pang - Initial API and implementation
 *****************************************************************/
var gc = gc || {};
gc.databind = gc.databind || {};

(function() // closure for private static methods and data.
{
	var generateID = function( fmt ) {
	   		var d = new Date().getTime();
	   	    var uuid = fmt.replace(/[xy]/g, function(c) {
	   	        var r = (d + Math.random()*16)%16 | 0;
	   	        d = Math.floor(d/16);
	   	    	return (c=='x' ? r : (r&0x3|0x8)).toString(16);
	   	    });
 		return uuid;
	};

	// IBM QuickStart View
	var QuickStartView = function( name, transport ) 
	{
	  	gc.databind.AbstractMqttView.call( this, name, transport );
	};
	
	QuickStartView.prototype = new gc.databind.AbstractMqttView();
	
	QuickStartView.prototype.formatTopicName = function( devID )
  	{
		return "iot-2/type/+/id/" + devID + "/evt/+/fmt/json";
  	}
  	
	QuickStartView.prototype.numberOfDotInTopicName = function( )
	{
		return 2;
	}

	QuickStartView.prototype.substitueMacros = function( prop )
	{
		gc.databind.AbstractMqttView.prototype.substitueMacros.call( this, prop );
		if ( prop.client !== undefined )
		{
			for( attr in prop.client )
			{
				if ( prop.client.hasOwnProperty(attr) ) {
					if ( this.isMacro( prop.client[attr] ))
					{
						prop.client[attr] = this.replaceMacro( prop.client[attr] );
					}
				}
			}
		}
	}

	QuickStartView.prototype.defaultProperties = function ()
	{
		return { devices : [ { name: '', id : '' } ] };
	};
	
	QuickStartView.prototype.subscribeDevice = function( device )
	{
		if ( device.id && device.id !== '')
		{
	   		this.subscribe( this.formatTopicName( device.id ), { timeout: 3, qos: 0 } );
		}
	};
	
	QuickStartView.prototype.subscribeDevices = function(devices)
	{
   		for( var i in devices )
		{
   			this.subscribeDevice( devices[i] );
		}
	};
	
	QuickStartView.prototype.parseMessage = function( message )
	{
	 	var parts = message.destinationName.split('/');
		var eventDetail;
			try
			{
				eventDetail = JSON.parse( message.payloadString );
			}
			catch( e )
			{
				eventDetail = { d : message.payloadString };
			}
		return { 
			devID : parts[4], 
			devTyp : parts[2], 
			event : { 
				name : parts[6], 
				detail : eventDetail['d']
			}
		};
	}
	
	QuickStartView.prototype.deviceLabel = function( cap, plural )
  	{
  		cap = cap || false;
  		plural = plural || false;
  		var label = cap ? "D" : "d"
  		label += "evice";
  		label += plural ? "s" : "";
  		return label;
  	}
  	
	QuickStartView.prototype.hasNoDevice = function(devices)
	{
		for ( i in devices )
		{
			var device = devices[i];
			if ( device.name && device.name !== '' && device.id && device.id !== '')
			{
				return false;
			}
		}
  		return true;
	}
	
	QuickStartView.prototype.connect = function(prop)
	{
		var conn = {
				useSSL : true,
				host : 'quickstart.messaging.internetofthings.ibmcloud.com',
				port : 443,
				clientid : "a:quickstart:" + generateID( 'tixxxxx' ),
				onConnect: onConnect,
			   	onMessageArrived: onMessageArrived				
		   };
 		
		if (!this.hasNoDevice(prop.devices))
		{
			this.connectTransport(conn);
			this.log( "Connecting to IOTF QuickStart ..." );
		}
   		var that = this;
   		function onConnect() 
   		{
   			that.subscribeDevices(prop.devices);
		}
		function onMessageArrived(message) {
			that.updateDevices( prop.devices, that.parseMessage( message ) );
		};
	};
	
    QuickStartView.prototype.updateDevice = function( device, i, msg )
    {
		this.setTopic( device.name + '.' + msg.event.name, msg.event.detail );
		this.logMsgArrival( device.name + "|" + msg.event.name, msg.event.detail );
    };

	QuickStartView.prototype.updateDevices = function( devices, msg )
	{
		for( var i in devices )
		{
			if ( devices[i].id == msg.devID )
			{
                this.updateDevice( devices[i], i, msg);
                return true;
 			}
		}
		return false;
	};
	
	/*****************************************************************/

	var QuickStartDesignView = function( name, dialog, transport ) 
	{
	  	gc.databind.AbstractMqttDesignView.call( this, name, dialog, transport );
	  	var that = this;
		
		this.onDeviceAdded = function ( event, detail, sender ) {
	  		if ( that.dialog.devices.list.length > 1 )
	  		{
				var last = that.dialog.devices.list.length -1;
				if ( that.dialog.devices.list[last].name !== undefined )
				{
					that.dialog.set('devices.list.0.name', that.dialog.devices.list[last].name + last.toString());
				}
				else
				{
					that.dialog.set('devices.list.0.name', 'dev' + last.toString());
				}
	  		}
		};
		
		this.onDeviceChanged = function ( event, detail, sender ) {
	  		var index = event.detail.index;
	  		var oldvalue = event.detail.oldValue;
	  		var newvalue = event.detail.newValue;
	  		
	  		if ( oldvalue.id === newvalue.id )
	  		{
	  			newvalue.name = newvalue.name.trim();
	  			if ( newvalue.name === '' )
	  			{
					that.dialog.set('devices.list.' + index + '.name', oldvalue.name);
		  			that.dialog.showError( that.deviceLabel( true ) + ' name cannot be null.')	  				
	  			}
	  		}
	  		else if ( oldvalue.name === newvalue.name )
	  		{
	  			newvalue.id = newvalue.id.trim();
	  			if ( newvalue.id === '' )
	  			{
					that.dialog.set('devices.list.' + index + '.id', oldvalue.id);
		  			that.dialog.showError(that.deviceLabel( true ) +' ID cannot be null.')	  				
	  			}
	  		}
	  		if ( oldvalue.id != '' )
	  		{
	  			that.unsubscribeDevice( oldvalue );
	  		}
	  		if ( newvalue.id != '' )
	  		{
	  			that.subscribeDevice( newvalue );
	  		}
	   	};
		
		this.onDeviceRemoved = function ( event, detail, sender ) {
	  		var device = event.detail.device;
	  		if ( that.dialog.devices.list.length === 0 )
	  		{
				that.dialog.set('devices.list.0', device);
 	  			that.dialog.showError('At least one ' + that.deviceLabel() + ' needs to be defined.')
	  		}
	  		else if (device.id != '')
	  		{
	  			that.unsubscribeDevice( device );
	  		}
		};
	}
	
	QuickStartDesignView.prototype = new gc.databind.AbstractMqttDesignView();

	QuickStartDesignView.prototype.formatTopicName = QuickStartView.prototype.formatTopicName;
	QuickStartDesignView.prototype.deviceLabel = QuickStartView.prototype.deviceLabel;
	QuickStartDesignView.prototype.numberOfDotInTopicName = QuickStartView.prototype.numberOfDotInTopicName;
	QuickStartDesignView.prototype.substitueMacros = QuickStartView.prototype.substitueMacros;
	QuickStartDesignView.prototype.defaultProperties = QuickStartView.prototype.defaultProperties;
	QuickStartDesignView.prototype.subscribeDevice = QuickStartView.prototype.subscribeDevice;
	QuickStartDesignView.prototype.subscribeDevices = QuickStartView.prototype.subscribeDevices;
	QuickStartDesignView.prototype.parseMessage = QuickStartView.prototype.parseMessage;
	QuickStartDesignView.prototype.connect = QuickStartView.prototype.connect;

	QuickStartDesignView.prototype.unsubscribeDevice = function( device )
	{
		if ( device.id && device.id !== '')
		{
	   		this.unsubscribe( this.formatTopicName( device.id ), { timeout: 3, qos: 0 } );
		}
	};
	
	QuickStartDesignView.prototype.hasNoDevice = function(devices)
	{
		if (QuickStartView.prototype.hasNoDevice( devices ))
		{
	  		this.dialog.showError('At least one ' + this.deviceLabel() + ' with non null ID and name needs to be defined.');
			return true;
		}
  		return false;
	}
	
	QuickStartDesignView.prototype.addEventListeners = function( )
	{
		gc.databind.AbstractMqttDesignView.prototype.addEventListeners.call( this );
		this.dialog.addEventListener( 'deviceAdded', this.onDeviceAdded );
		this.dialog.addEventListener( 'deviceChanged', this.onDeviceChanged );
		this.dialog.addEventListener( 'deviceRemoved', this.onDeviceRemoved );
	};

	QuickStartDesignView.prototype.removeEventListeners = function( )
	{
		gc.databind.AbstractMqttDesignView.prototype.removeEventListeners.call( this );
		this.dialog.removeEventListener( 'deviceAdded', this.onDeviceAdded );
		this.dialog.removeEventListener( 'deviceChanged', this.onDeviceChanged );
		this.dialog.removeEventListener( 'deviceRemoved', this.onDeviceRemoved );
	};

	QuickStartDesignView.prototype.initializeDialog = function( properties )
	{
		gc.databind.AbstractMqttDesignView.prototype.initializeDialog.call( this, properties );
		this.dialog.set('connection.visible',false);
		this.dialog.set('connection.expanded',false);
		this.dialog.set('client.visible',false);
		this.dialog.set('client.expanded',false);
		this.dialog.set('devices.heading',this.deviceLabel( true, true ));
		this.dialog.set('devices.modeVisible',false);
		this.dialog.set('devices.canSelectDevices',false);
		this.dialog.set('console.canSendCmd',false);
	};
	
	QuickStartDesignView.prototype.createDialogRow = function( rowProperties )
	{
		return {
			name : rowProperties.name, 
			id : rowProperties.id, 
			mode: 0,
			typ : rowProperties.typ,  
			events : rowProperties.events  
		};
	};

	QuickStartDesignView.prototype.extractFromDialogRow = function( row )
	{
		return {
			name : row.name, 
			id : row.id, 
			typ : row.typ,  
			events : row.events  
		};
	};

	QuickStartDesignView.prototype.extractPropertiesFromDialog = function()
	{
		var deviceList = 
			gc.databind.AbstractMqttDesignView.prototype.extractPropertiesFromDialog.call( this );
		return { devices : deviceList }; 
	};
	
    QuickStartDesignView.prototype.updateDevice = function( device, i, msg )
    {
            QuickStartView.prototype.updateDevice.call(this,device, i, msg);
    		device.typ = msg.devTyp;
    		// turn on the connection indicator
    		this.dialog.set('devices.list.'+i+'.active', true);
            if ( device.events === undefined )
            {
                    this.needToUpdateEndpoints = true;
                    device.events = {};
            }
            else if ( !device.events.hasOwnProperty(msg.event.name) )
            {
                    this.needToUpdateEndpoints = true;
            }
            device.events[msg.event.name] = msg.event.detail;
    };

    QuickStartDesignView.prototype.updateDevices = function( devices, msg )
    {
            // devices are from model not from dialog.
            this.needToUpdateEndpoints = false;
            var found = QuickStartView.prototype.updateDevices.call(this, this.dialog.devices.list, msg);
            if ( this.needToUpdateEndpoints )
            {
            	this.updateBindingEndpoints( this.dialog.devices.list );
            }
            return found;
    };

	/*****************************************************************/

	// IBM IOTF View
	var IotfView = function( name, transport ) 
	{
		QuickStartView.call( this, name, transport );
	};
	
	IotfView.prototype = new QuickStartView();
	
	IotfView.prototype.defaultProperties = function ()
	{
		var prop = {
				client : {
		 				org_id : '',
						api_key : '',
						api_auth_token : ''
					},
				devices : []
		};
		return prop;
	};
	
	IotfView.prototype.doConnect = function(client, onConn, onConnLost, onMsgArrived, onMsgDelivered)
	{
		var conn = {
				useSSL : true,
				host : client.org_id + '.messaging.internetofthings.ibmcloud.com',
				port : 443,
				clientid : "a:" + client.org_id + ":" + generateID( 'tixxxxx' ),
				userName: client.api_key,
				password: client.api_auth_token,
				onConnect: onConn,
				onConnectionLost: onConnLost,
			   	onMessageArrived: onMsgArrived,				
			   	onMessageDelivered: onMsgDelivered				
		   };
		
		this.connectTransport(conn);
		this.log( "Connecting to IOTF Foundation ..." );
	}

	IotfView.prototype.connect = function(prop)
	{
   		var that = this;
		this.doConnect(prop.client, onConnect, onConnectionLost, onMessageArrived, onMessageDelivered);
   		function onConnect() 
   		{
    		that.subscribeDevices(prop.devices);
		}
		function onMessageArrived(message) {
			that.updateDevices( prop.devices, that.parseMessage( message ) );
		};
   		function onConnectionLost() 
   		{
 		}
		function onMessageDelivered(message) {
			var parts = message.destinationName.split('/');
			var id = parts[4];
			var cmd = parts[6];
   			for( var i in prop.devices )
			{
   				if ( prop.devices[i].id == id)
   				{
   					that.log( "Command '" + cmd + "' has been sent to " + prop.devices[i].name );
   				}
			}
		}
	};
	
	/*****************************************************************/

	var IotfDesignView = function( name, dialog, transport ) 
	{
		QuickStartDesignView.call( this, name, dialog, transport );
		
		var that = this;
		
		this.onCommitDeviceSelection = function ( event, detail, sender ) {
			var selection = event.detail.selection;
			for( var i in selection )
			{
				if ( selection[i].selected )
				{
					selection[i].name = 'dev' + that.dialog.devices.list.length.toString();
	        		that.dialog.splice('devices.list', that.dialog.devices.list.length, 0, selection[i]);
				}
			}
			var found;
			do
			{
				found = false;
				for( var i in selection )
				{
					if ( selection[i].selected )
					{
		        		that.dialog.splice('devices.available', i, 1);
		        		found = true;
		        		break;
					}
				}
			}
			while(found);
	   	};
	};
	
	IotfDesignView.prototype = new QuickStartDesignView();
	IotfDesignView.prototype.doConnect = IotfView.prototype.doConnect;
	
	IotfDesignView.prototype.initializeDialog = function( properties )
	{
		QuickStartDesignView.prototype.initializeDialog.call( this, properties );
		this.dialog.set('client.visible',true);
		this.dialog.set('client.expanded', true);
		this.dialog.set('client.organization', properties.client.org_id);
		this.dialog.set('client.id', properties.client.api_key);
		this.dialog.set('client.password', properties.client.api_auth_token);
		this.dialog.set('client.organizationLabel','Organization ID');
		this.dialog.set('client.idLabel','API Key');
		this.dialog.set('client.passwordLabel','API Auth Token');
		this.dialog.set('console.canSendCmd', true);
	};
	
	IotfDesignView.prototype.extractPropertiesFromDialog = function()
	{
		var prop = QuickStartDesignView.prototype.extractPropertiesFromDialog.call( this );
		prop.client = {
				org_id : this.dialog.client.organization,
				api_key : this.dialog.client.id,
				api_auth_token : this.dialog.client.password
			};
		return prop;
	};
	
	IotfDesignView.prototype.defaultProperties = IotfView.prototype.defaultProperties;
	
	IotfDesignView.prototype.addAvailableDevice = function ( device, checkDeviceList )
	{
		if ( checkDeviceList )
		{
			for ( var i in this.dialog.devices.list )
			{
				if ( this.dialog.devices.list[i].id == device.id )
				{
					return;
				}	
			}
		}
		for ( var i in this.dialog.devices.available )
		{
			if ( this.dialog.devices.available[i].id == device.id )
			{
 				if ( device.hasOwnProperty('active') )
 				{
 					this.dialog.set('devices.available.'+i+'.active', device.active);
 				}
 				else if ( device.hasOwnProperty('detail') )
 				{
 					this.dialog.set('devices.available.'+i+'.detail', device.detail);
 				}
 				else if ( device.hasOwnProperty('events') )
 				{
 					this.dialog.set('devices.available.'+i+'.detail', device.events)
 				}
				return;
			}	
		}
		this.dialog.splice('devices.available', this.dialog.devices.available.length, 0, device);
	}
	
	IotfDesignView.prototype.connect = function(prop)
	{
   		var that = this;
		this.doConnect(prop.client, onConnect, onConnectionLost, onMessageArrived, onMessageDelivered);
   		function onConnect() 
   		{
   			// auto device discovery in designer mode
			that.dialog.set('devices.canSelectDevices', true);
			that.dialog.splice('devices.available',0, that.dialog.devices.available.splice.length);
   			that.subscribe( "iot-2/type/+/id/+/evt/+/fmt/json" );

	   		var httpReq = new XMLHttpRequest();
   			httpReq.onreadystatechange = function() {
   				if ( httpReq.readyState == 4 && httpReq.status == 200 )
   				{
   					try
   					{
   	   					var registeredDevices = JSON.parse( httpReq.response );
   	   					for( var i in registeredDevices)
   	   					{
	   	   					var device = { 
	   	   						id : registeredDevices[i].id,
	   	   						typ : registeredDevices[i].type, 
	   	   						mode: 0,
	   	   						desc: "Type:" + registeredDevices[i].type,
	   	   						detail: registeredDevices[i] };
	  	   					that.addAvailableDevice( device, true );
   	   					}
   					}
   					catch(e)
   					{
   						console.log( e );
   					}
   				}
   			}
  	  		var url = window.location.protocol + "//" + window.location.hostname 
   	  			+ "/mqtt/iotf/api/v0001/organizations/" + prop.client.org_id 
   	  			+ "/devices?username=" + encodeURIComponent(prop.client.api_key)
   	  			+ "&password=" + encodeURIComponent(prop.client.api_auth_token);
   			httpReq.open( 'GET', url, true ); 
   			httpReq.send();
   		}
   		function onConnectionLost() 
   		{
   		  	// auto device discovery in designer mode
			that.dialog.set('devices.canSelectDevices', false);
		}
		function onMessageArrived( message ) {
			var msg = that.parseMessage( message );
			if (!that.updateDevices( prop.devices, msg ))
			{
				var device = { 
					id : msg.devID,
					typ : msg.devTyp, 
					mode: 0,
					desc: "Type:" + msg.devTyp,
					active : true,
					events : {} };
				device.events[msg.event.name] = msg.event.detail;
				that.addAvailableDevice( device, false );
	   		}
		};
		function onMessageDelivered(message) {
			var parts = message.destinationName.split('/');
			var id = parts[4];
			var cmd = parts[6];
   			for( var i in that.dialog.devices.list )
			{
   				if ( that.dialog.devices.list[i].id == id)
   				{
   					that.log( "Command '" + cmd + "' has been sent to " + that.dialog.devices.list[i].name );
   				}
			}
		}
	};
	
	IotfDesignView.prototype.sendTopic = function( name, value )
	{
       	var parts = name.split('.');
       	var dev = parts[0];
       	var cmd = parts[1];
       	for( var i in this.dialog.devices.list )
		{
		  	if ( this.dialog.devices.list[i].name == dev )
		   	{
		   		this.log( "Publishing '" + name + "' ..." );
		   		var topic = "iot-2/type/" 
		   			+ this.dialog.devices.list[i].typ 
		   			+ "/id/" 
		   			+ this.dialog.devices.list[i].id 
		   			+ "/cmd/" + cmd + "/fmt/json"; 
		   		this.publish( topic, value, { qos : 0, retained : false });
		   		return;
		   	}
		}
		this.log( "Failed to publish: invalid device name '" + dev + "'" );      		
 	};
	
	IotfDesignView.prototype.addEventListeners = function( )
	{
		QuickStartDesignView.prototype.addEventListeners.call( this );
		this.dialog.addEventListener( 'commitDeviceSelection', this.onCommitDeviceSelection );
	};

	IotfDesignView.prototype.removeEventListeners = function( )
	{
		QuickStartDesignView.prototype.removeEventListeners.call( this );
		this.dialog.removeEventListener( 'commitDeviceSelection', this.onCommitDeviceSelection );
	};

	/*****************************************************************/
	var MqttView = function( name, transport ) 
	{
		QuickStartView.call( this, name, transport );
	};
	
	MqttView.prototype = new QuickStartView();
	
	MqttView.prototype.deviceLabel = function( cap, plural )
  	{
  		cap = cap || false;
  		plural = plural || false;
  		var label = cap ? "T" : "t"
  		label += "opic";
  		label += plural ? "s" : "";
  		return label;
  	}
  	
	MqttView.prototype.formatTopicName = function( devID )
  	{
		return devID;
  	}
  	
	MqttView.prototype.substitueMacros = function( prop )
	{
		gc.databind.AbstractMqttView.prototype.substitueMacros.call( this, prop );
		for( attr in prop.connection )
		{
			if ( prop.connection.hasOwnProperty(attr) ) {
				if ( this.isMacro( prop.connection[attr] ))
				{
					prop.connection[attr] = this.replaceMacro( prop.connection[attr] );
				}
			}
		}
	}

	MqttView.prototype.defaultProperties = function ()
	{
		var prop = {
				connection : {
		 				host : '',
						port : '',
						ssl : false,
						clientid : '',
						username: '',
						password : ''
					},
				devices : []
		};
		return prop;
	};
	
	MqttView.prototype.parseMessage = function( message )
	{
	 	var name = message.destinationName;
		var eventDetail;
		try
		{
			eventDetail = JSON.parse( message.payloadString );
		}
		catch( e )
		{
			eventDetail = { d : message.payloadString };
		}
		return { 
			devID : name, 
			event : eventDetail
		};
	}
	
	MqttView.prototype.subscribeDevice = function( device )
	{
		if ( device.mode === 0 || device.mode == 2 )
		{
			QuickStartView.prototype.subscribeDevice.call( this, device );
		}
	};
	
	MqttView.prototype.updateDevice = function( device, i, msg )
    {
 		if ( device.mode === 0 || device.mode === 2 )
		{
			this.setTopic( device.name, msg.event );
			this.logMsgArrival( device.name, msg.event );			
		}
    };

	MqttView.prototype.doConnect = function(prop, onMessageDelivered)
	{
 		var conn = {
 				useSSL : prop.connection.ssl,
 				host : prop.connection.host,
 				port : prop.connection.port,
 				clientid : prop.connection.clientid,
 				userName: prop.connection.username,
 				password: prop.connection.password,
 				onConnect: onConnect,
 			   	onMessageArrived: onMessageArrived,				
 			   	onMessageDelivered: onMessageDelivered				
 	  	};
 	 		
 		if ( conn.clientid === undefined )
 		{
 			conn.clientid = generateID( 'tixxxxx' );
 		}
 			
 		this.connectTransport(conn);
 		this.log( 'Connecting to ' + conn.host + ':' + conn.port + ' ...' );
 		
 		var that = this;
 	   	function onConnect() 
 	   	{
 	   		that.subscribeDevices(prop.devices);
 		}
 		function onMessageArrived(message) {
 			that.updateDevices( prop.devices, that.parseMessage( message ) );
 		};
	}
	
	MqttView.prototype.logTopicSent = function(devices, id)
	{
 		for( var i in devices )
		{
   			if ( devices[i].id == id)
   			{
   				this.log( "Topic '" + devices[i].name + "' has been sent" );
   				break;
   			}
		}	
	}
	
 	MqttView.prototype.connect = function(prop)
	{
 		this.doConnect(prop, onMessageDelivered);
  		var that = this;
		function onMessageDelivered( message ) {
			that.logTopicSent(prop.devices, message.destinationName);
 		}
	};
	
	/*****************************************************************/
	
	var MqttDesignView = function( name, dialog, transport ) 
	{
		QuickStartDesignView.call( this, name, dialog, transport );
	};
	
	MqttDesignView.prototype = new QuickStartDesignView();
	
	MqttDesignView.prototype.formatTopicName = MqttView.prototype.formatTopicName;
	MqttDesignView.prototype.deviceLabel = MqttView.prototype.deviceLabel;
	MqttDesignView.prototype.substitueMacros = MqttView.prototype.substitueMacros;
	MqttDesignView.prototype.defaultProperties = MqttView.prototype.defaultProperties;
	MqttDesignView.prototype.parseMessage = MqttView.prototype.parseMessage;
	MqttDesignView.prototype.doConnect = MqttView.prototype.doConnect;
	MqttDesignView.prototype.logTopicSent = MqttView.prototype.logTopicSent;
	MqttDesignView.prototype.subscribeDevice = MqttView.prototype.subscribeDevice;
	
	MqttDesignView.prototype.initializeDialog = function( properties )
	{
		gc.databind.AbstractMqttDesignView.prototype.initializeDialog.call( this, properties );
		this.dialog.set('connection.visible',true);
		this.dialog.set('connection.expanded',true);
		this.dialog.set('connection.host',properties.connection.host);
		this.dialog.set('connection.port',properties.connection.port);
		this.dialog.set('connection.ssl',properties.connection.ssl);
		this.dialog.set('client.visible',true);
		this.dialog.set('client.expanded',true);
		this.dialog.set('client.organizationLabel','ClientID');
		this.dialog.set('client.idLabel','Username');
		this.dialog.set('client.passwordLabel','Password');
		this.dialog.set('client.organization',properties.connection.clientid);
		this.dialog.set('client.id',properties.connection.username);
		this.dialog.set('client.password',properties.connection.password);
		this.dialog.set('console.canSendCmd',true);
		this.dialog.set('devices.heading','Topics');
		this.dialog.set('devices.modeVisible',true);
		this.dialog.set('devices.canSelectDevices',false);
	};
	
	MqttDesignView.prototype.createDialogRow = function( rowProperties )
	{
		return {
			name : rowProperties.name, 
			id : rowProperties.id, 
			mode: rowProperties.mode,
			typ : '',  
			events : rowProperties.events  
		};
	};

	MqttDesignView.prototype.extractFromDialogRow = function( row )
	{
		return {
			name : row.name, 
			id : row.id, 
			mode : row.mode,  
			events : row.events  
		};
	};

	MqttDesignView.prototype.extractPropertiesFromDialog = function()
	{
		var devicesList = 
			gc.databind.AbstractMqttDesignView.prototype.extractPropertiesFromDialog.call( this );
		var prop = { devices : devicesList };
		prop.connection = {
				host : this.dialog.connection.host,
				port : this.dialog.connection.port,
				ssl : this.dialog.connection.ssl,
				clientid : this.dialog.client.organization,
				username: this.dialog.client.id,
				password : this.dialog.client.password
			};
		return prop;
	};
	
	MqttDesignView.prototype.updateDevice = function( device, i, msg )
    {
		if ( device.mode === 0 || device.mode === 2 )
		{
			MqttView.prototype.updateDevice.call(this, device, i, msg);
			this.dialog.set('devices.list.'+i+'.active', true);
   			if ( device.events === undefined )
   			{
   				this.needToUpdateEndpoints = true;
   				device.events = msg.event;
   			}
		}
    };

	MqttDesignView.prototype.connect = function(prop)
	{
 		this.doConnect(prop, onMessageDelivered);
  		var that = this;
		function onMessageDelivered( message ) {
			that.logTopicSent(that.dialig.devices.list, message.destinationName);
 		}
	};
	
	MqttDesignView.prototype.sendTopic = function( name, value )
	{
       	for( var i in this.dialog.devices.list )
		{
       		var mode =  this.dialog.devices.list[i].mode;
       		var isSendable = (mode === 0 || mode === 1);
		  	if ( isSendable && this.dialog.devices.list[i].name === name )
		   	{
		   		this.log( "Publishing '" + name + "' ..." );
		   		this.publish( this.dialog.devices.list[i].id, value, { qos : 0, retained : false });
		   		return;
		   	}
		}
		this.log( "Failed to publish: invalid topic name '" + name + "'" );      		
	};
	
	/*****************************************************************/

	gc.databind.CreateMqttView = function( name, dialog ) 
	{
	 	var parts = name.split('.');
	 	var view = parts[0];
	 	var transport = parts.length > 1 ? parts[1] : "paho";
	 	if ( dialog )
	 	{
			if ( view === "quickstart" )
			{
				return new QuickStartDesignView(view, dialog, transport);
			}
			else if ( view === "iotf" )
			{
				return new IotfDesignView(view, dialog, transport);
			}
			else if ( view === 'mqtt' )
			{
				return new MqttDesignView(view, dialog, transport);				
			}				
	 	}
	 	else
	 	{
			if ( view === "quickstart" )
			{
				return new QuickStartView(view, transport);
			}
			else if ( view === "iotf" )
			{
				return new IotfView(view, transport);
			}
			else if ( view === 'mqtt' )
			{
				return new MqttView(view, transport);				
			}				
	 	}
		return undefined;
	};

}());
